#include <iostream>
#include <string.h>
#include <fstream>
#include <cstdlib>
#include <math.h>
#include <time.h>
#include <time.h>

using namespace std;

#define MAIN
#include "ldraw.hpp"

void pause(void)
{
	cout << "Press <Enter> to quit" << endl;
	getchar();
}
void usage(void)
{
	cout << "SymSplitter [options] Infile, [SplitterFile] Outfile" << endl;
	cout << "-p <val>: precision (default 0.001)" << endl;
	cout << "-x+: split by plane x=0, keep elements x>0" << endl;
	cout << "-x++ <val>: split by plane x=val, keep elements x>val" << endl;
	cout << "-x-: split by plane x=0, keep elements x<0" << endl;
	cout << "-x-- <val>: split by plane x=val, keep elements x<val" << endl;
	cout << "-y+, -y++, -y-, -y--,  -z+, -z++, -z--: ditto for y or z" << endl;
	cout << "-da: delete elements across splitter plane" << endl;
	cout << "-di: delete elements completely In splitter plane" << endl;
	cout << "-dn: delete elements not in or across splitter plane" << endl;
	cout << "-d0: no elements deleted, only snaps to splitter plane" << endl;
	cout << "-i: invert split direction" << endl;
	cout << "-c: color elements across or along splitter plane" << endl;
	cout << "-ca: cut elements across splitting plane" << endl;
	cout << "-v: verify self-symmetry of elements across splitter plane" << endl;
	cout << "-s: expand subfiles" << endl;
	cout << "-l \"path\": define ldraw search path" << endl;
	pause();
}

int main(int argc, char *argv[])
{
	printf("\nLdraw SymSplitter v1.2 - by Philo");
	printf("\n---------------------------------\n");


	if(argc<4) 
	{
		usage();
		return 1;
	}

	LdPath* path=new LdPath();
	// Get input file path
	string curpath=argv[argc-2];
	size_t pathlen = curpath.find_last_of("\\");
	if(pathlen == string::npos) curpath = ".\\"; else curpath.resize(pathlen+1);
	path->addPath(curpath);
	string derivedpath=curpath;
	path->addPath(derivedpath.append("p\\"));
	derivedpath=curpath;
	path->addPath(derivedpath.append("s\\"));

	Fldraw sep;
	bool Split=false;
	LdTriangle *t;
	int i=1;
	double Offset;
	bool isOffset;
	Point *p1, *p2, *p3;
	while(argv[i][0] == '-')
	{
		switch (argv[i][1])
		{
		case 'p':
			i++;
			sscanf(argv[i], "%lf",&LdPrecision);
			break;
		case 'c':
			switch (argv[i][2])
			{
			case 'a':
				CutAcross=true;
				cout << "Cut elements by splitter plane" << endl;
				break;
			case 0:
				Coloring=true;
				break;
			default:
				usage();
				return 1;
			}
			break;


		case 's':
			Expand=true;
			Inlined=true;
			SubComments=false;
			break;

		case 'v':
			Verify=true;
			break;

		case 'i':
			InvertSplit=true;
			break;

		case 'l':
			i++;
			curpath = argv[i];
			if(curpath.at(curpath.size()-1) != '\\') 
			{
				curpath.append("\\");
			}
			derivedpath=curpath;
			path->addPath(derivedpath.append("p\\"));
			derivedpath=curpath;
			path->addPath(derivedpath.append("parts\\"));
			path->addPath(derivedpath.append("s\\"));
			curpath.append("Unofficial\\");
			derivedpath=curpath;
			path->addPath(derivedpath.append("p\\"));
			derivedpath=curpath;
			path->addPath(derivedpath.append("parts\\"));
			path->addPath(derivedpath.append("s\\"));

			break;
		case 'd':
			switch (argv[i][2])
			{
			case 'a':
				Across=true;
				cout << "Elements across splitter plane deleted" << endl;
				break;
			case 'i':
				InSplit=true;
				cout << "Elements completely in splitter plane deleted" << endl;
				break;
			case 'n':
				Middle=true;
				cout << "Elements not across or in splitter plane deleted" << endl;
				break;
			case '0':
				Nothing=true;
				cout << "No elements deleted, only snaps to splitter plane" << endl;
				break;
			default:
				usage();
				return 1;
			}
			break;
		case 'x':
			isOffset=false;
			Offset=0;
			if(strlen(argv[i])>3)
			{
				if((argv[i][3] == '-')||(argv[i][3] == '+')) 
				{
					isOffset=true;
					sscanf(argv[i+1], "%lf",&Offset);
				}
				else
				{
					usage();
					return 1;
				}
			}

			switch (argv[i][2])
			{
			case '-':
				p1=new Point(Offset,0,0);
				p2=new Point(Offset,10,0);
				p3=new Point(Offset,0,10);
				t=new LdTriangle(*p1, *p2, *p3, 16);
				sep.pushElement(t);
				Split=true;
				break;
			case '+':
				p1=new Point(Offset,0,0);
				p3=new Point(Offset,10,0);
				p2=new Point(Offset,0,10);
				t=new LdTriangle(*p1, *p2, *p3, 16);
				sep.pushElement(t);
				Split=true;
				break;
			default:
				usage();
				return 1;
			}
			if(isOffset) i++;
			break;
		case 'y':
			isOffset=false;
			Offset=0;
			if(strlen(argv[i])>3)
			{
				if((argv[i][3] == '-')||(argv[i][3] == '+')) 
				{
					isOffset=true;
					sscanf(argv[i+1], "%lf",&Offset);
				}
				else
				{
					usage();
					return 1;
				}
			}

			switch (argv[i][2])
			{
			case '-':
				p1=new Point(10,Offset,0);
				p2=new Point(0,Offset,0);
				p3=new Point(0, Offset,10);
				t=new LdTriangle(*p1, *p2, *p3, 16);
				sep.pushElement(t);
				Split=true;
				break;
			case '+':
				p1=new Point(10,Offset,0);
				p3=new Point(0,Offset,0);
				p2=new Point(0, Offset,10);
				t=new LdTriangle(*p1, *p2, *p3, 16);
				sep.pushElement(t);
				Split=true;
				break;
			default:
				usage();
				return 1;
			}
			if(isOffset) i++;
			break;

		case 'z':
			isOffset=false;
			Offset=0;
			if(strlen(argv[i])>3)
			{
				if((argv[i][3] == '-')||(argv[i][3] == '+')) 
				{
					isOffset=true;
					sscanf(argv[i+1], "%lf",&Offset);
				}
				else
				{
					usage();
					return 1;
				}
			}

			switch (argv[i][2])
			{
			case '-':
				p1=new Point(0,10,Offset);
				p2=new Point(10,0,Offset);
				p3=new Point(0,0,Offset);
				t=new LdTriangle(*p1, *p2, *p3, 16);
				sep.pushElement(t);
				Split=true;
				break;
			case '+':
				p1=new Point(0,10,Offset);
				p3=new Point(10,0,Offset);
				p2=new Point(0,0,Offset);
				t=new LdTriangle(*p1, *p2, *p3, 16);
				sep.pushElement(t);
				Split=true;
				break;
			default:
				usage();
				return 1;
			}
			if(isOffset) i++;
			break;


		default:
			usage();
			return 1;
		}
		i++;
	}

	if (InvertSplit)
	{
		sep.InvertWinding();
	}
	if(Verify) Coloring = false;
	i=argc-i;
	if( i > 3 || i < 2)
	{
		printf("Wrong number of arguments\n");
		usage();
		return 1;
	}
	if(!Split && i==2)
	{	
		printf("No split specified\n");
		usage();
		return 1;
	}

	printf("Precision %g\n\n", LdPrecision);

	Fldraw in;
	if(!in.ldLoad (argv[argc-i]))
	{
		cout << argv[argc-i] << " not found" << endl;
		pause();
		return 2;
	}

	if(i==3)
	{
		if(!sep.ldLoad (argv[argc-2]))
		{
			cout << argv[argc-2] << " not found" << endl;
			pause();
			return 2;
		}
	}
	if (Expand)
	{
		cout << "\nExpanding subfiles\n";
		cout << "LDraw subfiles search locations:\n" << path->toString().c_str() << endl;
		in.ldExpand(path);
		cout << endl;
	}

	in.separate(&sep);

	if(!in.ldSave(argv[argc-1]))
	{
		cout <<" Can't create" << argv[argc-1] << endl;
		pause();
		return 2;
	}

	pause();
	return 0;
}

